(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory(require('react')) :
  typeof define === 'function' && define.amd ? define(['react'], factory) :
  (global['react-filter-control'] = factory(global.React));
}(this, (function (React) { 'use strict';

  var React__default = 'default' in React ? React['default'] : React;

  function styleInject(css, ref) {
    if ( ref === void 0 ) ref = {};
    var insertAt = ref.insertAt;

    if (!css || typeof document === 'undefined') { return; }

    var head = document.head || document.getElementsByTagName('head')[0];
    var style = document.createElement('style');
    style.type = 'text/css';

    if (insertAt === 'top') {
      if (head.firstChild) {
        head.insertBefore(style, head.firstChild);
      } else {
        head.appendChild(style);
      }
    } else {
      head.appendChild(style);
    }

    if (style.styleSheet) {
      style.styleSheet.cssText = css;
    } else {
      style.appendChild(document.createTextNode(css));
    }
  }

  var css = ".fc {\n    font-family: \"Helvetica Neue\",Helvetica,\"Segoe UI\",Verdana,\"san-serif\";\n}\n.fc-dropdownmenu-button {\n    margin-right: 3px;\n    margin-top: 3px;\n}\n.fc-dropdownmenu-contextmenu {\n    position: fixed;\n    background-color: white;\n    z-index: 20002;\n    padding: 5px;\n    box-shadow: 0 1px 10px #0005;\n}\n.fc-dropdownmenu-contextmenu-item {\n    padding: 5px;\n    cursor: pointer;\n}\n.fc-dropdownmenu-contextmenu-background {\n    position: fixed;\n    width: 100%;\n    height: 100%;\n    z-index: -1;\n    left: 0;\n    top: 0;\n}\n.fc-dropdownmenu-button {\n    padding: 7px 10px;\n    opacity: .8;\n    cursor: pointer;\n}\n.fc-dropdownmenu-button:hover {\n    opacity: 1;\n}\n.fc-button-color-secondary {\n    color: blue;\n}\n.fc-button-color-primary {\n    color: blue;\n}\n.fc-group-settings-row {\n    display: flex;\n}\n.fc-button-add:before {\n    margin: 3px;\n    width: 30px;\n    height: 30px;\n    content: '+';\n    font-size: 26px;\n    display: inline-flex;\n    flex-direction: column;\n    justify-content: center;\n    text-align: center;\n    color: green;\n    cursor: pointer;\n}\n.fc-button-remove:before {\n    margin: 3px;\n    width: 30px;\n    height: 30px;\n    content: '\\00d7';\n    font-size: 26px;\n    display: inline-flex;\n    flex-direction: column;\n    justify-content: center;\n    text-align: center;\n    color: red;\n    cursor: pointer;\n}\n.fc-value-editor {\n    margin: 3px;\n    padding: 0 3px;\n}";
  styleInject(css);

  const AddButton = ({
    onClick
  }) => React__default.createElement("div", null, React__default.createElement("div", {
    className: "fc-group-button-icon fc-button-add",
    onClick: onClick,
    onKeyDown: onClick,
    tabIndex: "0",
    "aria-label": "Add",
    role: "button"
  }));

  const RemoveButton = ({
    onClick
  }) => React__default.createElement("div", {
    className: "fc-group-button-icon fc-button-remove",
    onClick: onClick,
    onKeyDown: onClick,
    tabIndex: "0",
    "aria-label": "Remove",
    role: "button"
  });

  const Button = ({
    text,
    color,
    onClick
  }) => React__default.createElement("div", {
    className: `fc-dropdownmenu-button fc-button-color-${color || 'default'}`,
    "aria-haspopup": "true",
    onClick: onClick,
    onKeyDown: onClick,
    tabIndex: "0",
    role: "button"
  }, React__default.createElement("span", null, text));

  const Input = ({
    value,
    onChange
  }) => React__default.createElement("input", {
    value: value,
    onChange: onChange,
    className: "fc-value-editor"
  });

  const Menu = ({
    visible,
    handleClose,
    menuItems,
    handleMenuItemClick,
    textField,
    keyField,
    activeIndex
  }) => visible ? React__default.createElement("div", {
    className: "fc-dropdownmenu-contextmenu"
  }, menuItems.map((item, index) => {
    const onClick = () => handleMenuItemClick(index);

    return React__default.createElement("div", {
      className: "fc-dropdownmenu-contextmenu-item",
      key: keyField ? item[keyField] : index,
      selected: index === activeIndex,
      onClick: onClick,
      onKeyDown: onClick,
      tabIndex: "0",
      role: "button"
    }, textField ? item[textField] : item);
  }), React__default.createElement("div", {
    className: "fc-dropdownmenu-contextmenu-background",
    onClick: handleClose,
    onKeyDown: handleClose,
    role: "button",
    tabIndex: "0"
  })) : React__default.createElement("div", null);
  const FCContext = React__default.createContext({
    AddButton,
    RemoveButton,
    Button,
    Input,
    Menu
  });
  const {
    Consumer
  } = FCContext;

  class DropDownMenu extends React.Component {
    constructor(props) {
      super(props);
      this.state = {
        anchorEl: null
      };
      this.handleButtonClick = this.handleButtonClick.bind(this);
      this.handleMenuItemClick = this.handleMenuItemClick.bind(this);
      this.handleClose = this.handleClose.bind(this);
    }

    handleButtonClick(event) {
      this.setState({
        anchorEl: event.currentTarget
      });
    }

    handleMenuItemClick(index) {
      const {
        menuItems,
        keyField,
        onMenuItemClick
      } = this.props;
      const selected = menuItems[index];
      onMenuItemClick(keyField ? selected[keyField] : selected);
      this.setState({
        anchorEl: null
      });
    }

    handleClose() {
      this.setState({
        anchorEl: null
      });
    }

    renderButton(children) {
      return React__default.Children.map(children, child => React__default.cloneElement(child, {
        onClick: this.handleButtonClick
      }));
    }

    render() {
      const {
        activeIndex,
        color,
        menuItems,
        textField,
        keyField,
        children
      } = this.props;
      const {
        anchorEl
      } = this.state;
      return React__default.createElement("span", {
        className: "fc-dropdownmenu"
      }, children ? this.renderButton(children) : React__default.createElement(Consumer, null, value => React__default.createElement(value.Button, {
        text: textField ? menuItems[activeIndex][textField] : menuItems[activeIndex],
        color: color,
        onClick: this.handleButtonClick
      })), React__default.createElement(Consumer, null, value => React__default.createElement(value.Menu, {
        handleMenuItemClick: this.handleMenuItemClick,
        handleClose: this.handleClose,
        visible: anchorEl != null,
        menuItems: menuItems,
        textField: textField,
        keyField: keyField,
        activeIndex: activeIndex
      })));
    }

  }

  function deepCopy (obj) {
    return JSON.parse(JSON.stringify(obj));
  }

  var css$1 = ".fc-condition {\n    display: flex;\n}\n\n.fc-condition input {\n    width: 120px;\n}";
  styleInject(css$1);

  const Condition = ({
    fields,
    removeClick,
    onConditionValueChanged,
    value
  }) => {
    const getActiveField = conditionValue => {
      const index = fields.findIndex(field => field.name === conditionValue.field);
      const activeField = fields[index];
      const activeOperatorIndex = activeField.operators.findIndex(operator => operator.name === conditionValue.operator);
      return {
        index,
        activeOperatorIndex,
        operators: activeField.operators,
        value: conditionValue.value
      };
    };

    const updateCondition = (propertyName, newValue) => {
      const conditionValue = deepCopy(value);
      conditionValue[propertyName] = newValue;

      if (propertyName === 'field') {
        const activeField = getActiveField(conditionValue);
        conditionValue.operator = activeField.operators[0].name;
        conditionValue.value = '';
      }

      if (propertyName === 'operator') {
        conditionValue.value = '';
      }

      return conditionValue;
    };

    const handleValueChanged = e => onConditionValueChanged(updateCondition('value', e.target.value));

    const handleFieldChanged = newValue => onConditionValueChanged(updateCondition('field', newValue));

    const handleOperatorChanged = newValue => onConditionValueChanged(updateCondition('operator', newValue));

    const activeField = getActiveField(value);
    return React__default.createElement("div", {
      className: "fc-condition"
    }, React__default.createElement(Consumer, null, consumer => React__default.createElement(consumer.RemoveButton, {
      onClick: removeClick
    })), React__default.createElement(DropDownMenu, {
      textField: "name",
      keyField: "name",
      activeIndex: activeField.index,
      menuItems: fields,
      onMenuItemClick: handleFieldChanged
    }), React__default.createElement(DropDownMenu, {
      textField: "caption",
      color: "primary",
      keyField: "name",
      activeIndex: activeField.activeOperatorIndex,
      menuItems: activeField.operators,
      onMenuItemClick: handleOperatorChanged
    }), React__default.createElement(Consumer, null, consumer => React__default.createElement(consumer.Input, {
      value: activeField.value,
      onChange: handleValueChanged
    })));
  };

  var css$2 = "";
  styleInject(css$2);

  const FilterGroupSettingsRow = ({
    filterValue,
    fields,
    onFilterValueChanged,
    groups,
    removeClick
  }) => {
    const onGroupMenuItemClick = newGroupName => {
      const newFilterValue = deepCopy(filterValue);
      newFilterValue.groupName = newGroupName;
      onFilterValueChanged(newFilterValue);
    };

    const getNewKey = items => items.length ? Math.max(...items.map(o => o.key)) + 1 : 0;

    const addCondition = () => {
      const newFilterValue = deepCopy(filterValue);
      const key = getNewKey(newFilterValue.items);
      const field = fields[0];
      newFilterValue.items.push({
        key,
        field: field.name,
        operator: field.operators[0].name,
        value: field.defaultValue || ''
      });
      onFilterValueChanged(newFilterValue);
    };

    const addGroup = () => {
      const newFilterValue = deepCopy(filterValue);
      const key = getNewKey(newFilterValue.items);
      const group = groups[0];
      newFilterValue.items.push({
        key,
        groupName: group.name,
        items: []
      });
      onFilterValueChanged(newFilterValue);
    };

    const onAddMenuItemClick = name => name === 'addCondition' ? addCondition() : addGroup();

    const addMenuItems = [{
      name: 'addCondition',
      caption: 'Add Condition'
    }, {
      name: 'addGroup',
      caption: 'Add Group'
    }];
    const activeIndex = groups.findIndex(g => g.name === filterValue.groupName);
    return React__default.createElement("div", {
      className: "fc-group-settings-row"
    }, removeClick && React__default.createElement(Consumer, null, consumer => React__default.createElement(consumer.RemoveButton, {
      onClick: removeClick
    })), React__default.createElement(DropDownMenu, {
      color: "secondary",
      activeIndex: activeIndex,
      textField: "caption",
      keyField: "name",
      menuItems: groups,
      onMenuItemClick: onGroupMenuItemClick
    }), React__default.createElement(Consumer, null, value => React__default.createElement(DropDownMenu, {
      textField: "caption",
      activeIndex: 1,
      keyField: "name",
      menuItems: addMenuItems,
      onMenuItemClick: onAddMenuItemClick
    }, React__default.createElement(value.AddButton, null))));
  };

  var css$3 = ".fc-group-content {\n    padding-left: 20px;\n}\n\n.fc-group .fc-group-button-icon {\n    opacity: .6;\n    font-weight: 100;\n}\n\n.fc-group .fc-group-button-icon:hover {\n    opacity: .8;\n}";
  styleInject(css$3);

  const FilterGroup = ({
    fields,
    groups,
    filterValue,
    onFilterValueChanged,
    removeClick
  }) => {
    const onItemValueChanged = (index, newValue) => {
      const newFilterValue = deepCopy(filterValue);
      newFilterValue.items[index] = newValue;
      onFilterValueChanged(newFilterValue);
    };

    const removeItem = index => {
      const newFilterValue = deepCopy(filterValue);
      newFilterValue.items.splice(index, 1);
      onFilterValueChanged(newFilterValue);
    };

    return React__default.createElement("div", {
      className: "fc-group"
    }, React__default.createElement(FilterGroupSettingsRow, {
      filterValue: filterValue,
      fields: fields,
      onFilterValueChanged: onFilterValueChanged,
      groups: groups,
      removeClick: removeClick
    }), React__default.createElement("div", {
      className: "fc-group-content"
    }, filterValue.items.map((item, index) => {
      const filterValueHandler = value => onItemValueChanged(index, value);

      const removeClickHandler = () => removeItem(index);

      return item.groupName ? React__default.createElement(FilterGroup, {
        key: item.key,
        fields: fields,
        groups: groups,
        filterValue: item,
        onFilterValueChanged: filterValueHandler,
        removeClick: removeClickHandler
      }) : React__default.createElement(Condition, {
        key: item.key,
        value: item,
        fields: fields,
        onConditionValueChanged: filterValueHandler,
        removeClick: removeClickHandler
      });
    })));
  };

  class FilterControl extends React.Component {
    constructor(props) {
      super(props);
      const {
        filterValue
      } = this.props;
      this.state = {
        filterValue
      };
      this.onFilterValueChanged = this.onFilterValueChanged.bind(this);
    }

    onFilterValueChanged(filterValue) {
      const {
        onFilterValueChanged
      } = this.props;
      this.setState({
        filterValue
      });
      onFilterValueChanged(filterValue);
    }

    render() {
      const {
        fields,
        groups
      } = this.props;
      const {
        filterValue
      } = this.state;
      return React__default.createElement("div", {
        className: "fc"
      }, React__default.createElement(FilterGroup, {
        fields: fields,
        groups: groups,
        filterValue: filterValue,
        onFilterValueChanged: this.onFilterValueChanged
      }));
    }

  }

  return FilterControl;

})));
